
#Functions used in community phylogenetics analyses

#Function deg2rad to convert degrees to radians
deg2rad <- function(deg) return(deg*pi/180)

# Function to calculate the geodesic distance between two points specified by 
# radian latitude/longitude using the Haversine formula
# Ouputs distance between sites 1 and 2 as meters
gcd.hf <- function(long1, lat1, long2, lat2) {
  R <- 6371 # Earth mean radius [km]
  delta.long <- (long2 - long1)
  delta.lat <- (lat2 - lat1)
  a <- sin(delta.lat/2)^2 + cos(lat1) * cos(lat2) * sin(delta.long/2)^2
  c <- 2 * asin(min(1,sqrt(a)))
  d = (R * c)*1000
  return(d) # Distance in meters
}
#End function gcd.hf

# Function to calculate matrix of distances between each two sites
# INPUT: a data frame in which longs are in first column and lats in second column
# OUTPUT: a distance matrix (class dist) between all pairwise sites
# Output distances are in meters
CalcDists <- function(longlats) {
  name <- list(rownames(longlats), rownames(longlats))
  n <- nrow(longlats)
  z <- matrix(0, n, n, dimnames = name)
  for (i in 1:n) {
    for (j in 1:n) z[i, j]<-gcd.hf(long1 = deg2rad(longlats[i, 2]), 
                                   lat1 = deg2rad(longlats[i, 1]), long2 = deg2rad(longlats[j, 2]), 
                                   lat2 = deg2rad(longlats[j, 1]))
  }
  #z <- as.dist(z)
  return(z)
}
#End function CalcDists

#Function Subsample.comm.matrix to remove certain percentage of species from communities
#Use this function to account for sample size bias in turnover estimates
#Note this function is called within functions below
Subsample.comm.matrix=function(matrix,prop.remove.varied){
  #Gather names of sites with more than 7 observed species
  rich.sites<-names(which((rowSums(matrix != 0))>7)==TRUE)
  #Gather rows for sites that have fewer than 7 observed species
  subsamp.keep<-matrix[!rownames(matrix) %in% rich.sites, ]
  #Gather rows for sites that have more than 7 observed species
  subsamp.change<-matrix[rownames(matrix) %in% rich.sites, ]
  #Replace specified percentage of observations with zero
  samp.size<-length(subsamp.change[subsamp.change>0])
  samples<-c(rep(0,round(samp.size*prop.remove.varied)),
             rep(1,round(samp.size*(1-prop.remove.varied))))
  subsamp.change[subsamp.change>0] <- sample(samples,samp.size,replace=T)
  #Bind all sites together in a final matrix
  new.comm<-rbind(subsamp.keep,subsamp.change) 
  new.comm <- new.comm[ order(row.names(new.comm)), ]
  return(new.comm)
}

#Function phylo.beta to calculate phylo community distances
#Fxn also runs a multiple regression on dist matrices (paras~host+geo.dist+climate.dists)
phylo.beta = function(host.comm,host.BL.comm,paras.comm,
                      host.phylogeny,host.BL.phylogeny,paras.phylogeny,
                      Geo.dist,maxtemp.dist,mintemp.dist,precip.dist,
                      calculation,prop.remove){
  require(PhyloMeasures)
  require(vegan)
  require(ecodist)
  
  #remove specified percentage of species from well-sampled sites 
  #to account for potential sample size bias
  #percent to remove varies from none to the specified percentage
  props.remove<-runif(100,min=0,max=prop.remove)
  prop.remove.varied<-sample(props.remove,1,replace=T)
  paras.comm.sub<-Subsample.comm.matrix(paras.comm,prop.remove.varied)
  host.comm.sub<-Subsample.comm.matrix(host.comm,prop.remove.varied)
  host.BL.comm.sub<-Subsample.comm.matrix(host.BL.comm,prop.remove.varied)
  
  #identify all host and parasite trees and scale branch lengths
  randparas.tree<-sample(paras.phylogeny,size=1)[[1]]
  randparas.tree$edge.length=(randparas.tree$edge.length)/max(randparas.tree$edge.length)
  randhost.tree<-sample(host.phylogeny,size=1)[[1]]
  randhost.tree$edge.length=(randhost.tree$edge.length)/max(randhost.tree$edge.length)
  randhost.BL.tree<-sample(host.BL.phylogeny,size=1)[[1]]
  randhost.BL.tree$edge.length=(randhost.BL.tree$edge.length)/max(randhost.BL.tree$edge.length)
  
  #scale geographic and climate distance matrices
  Geo.dist<-Geo.dist/max(Geo.dist)
  maxtemp.dist<-maxtemp.dist/max(maxtemp.dist)
  mintemp.dist<-mintemp.dist/max(mintemp.dist)
  precip.dist<-precip.dist/max(precip.dist)
  
  #calculate community phylobeta diversity (either 'nt' or 'cd')
  if(calculation=="nt"){
    Paras.dist<-as.matrix(cdnt.averaged.query(randparas.tree,paras.comm.sub))
    host.dist<-as.matrix(cdnt.averaged.query(randhost.tree,host.comm.sub))
    host.BL.dist<-as.matrix(cdnt.averaged.query(randhost.BL.tree,host.BL.comm.sub))
  } 
  if(calculation=="cd"){
    Paras.dist<-as.matrix(cd.query(randparas.tree,paras.comm.sub))
    host.dist<-as.matrix(cd.query(randhost.tree,host.comm.sub))
    host.BL.dist<-as.matrix(cd.query(randhost.BL.tree,host.BL.comm.sub))
  }
  
  #Gather mean comm distances (corrected for geo.dist) as index of phylo-connectivity
  Host.means<-colMeans(host.dist)/scale(colMeans(Geo.dist))
  Host.BL.means<-colMeans(host.BL.dist)/scale(colMeans(Geo.dist))
  Paras.means<-colMeans(Paras.dist)/scale(colMeans(Geo.dist))
  reg.names<-row.names(host.comm)
  
  #Apply same rownames to all distance matrices for simultaneous identification
  rownames(Paras.dist)<-colnames(Paras.dist)<-rownames(paras.comm)
  rownames(host.dist)<-colnames(host.dist)<-rownames(paras.comm)
  rownames(host.BL.dist)<-colnames(host.BL.dist)<-rownames(paras.comm)
  rownames(Geo.dist)<-colnames(Geo.dist)<-rownames(paras.comm)
  rownames(maxtemp.dist)<-colnames(maxtemp.dist)<-rownames(paras.comm)
  rownames(mintemp.dist)<-colnames(mintemp.dist)<-rownames(paras.comm)
  rownames(precip.dist)<-colnames(precip.dist)<-rownames(paras.comm)
  
  #Identify sites with zero parasite observations
  cols.rem<-names(which(apply(Paras.dist,2,function(x)all(x==0))==T))
  
  #Use these sites to remove corresponding columns in distance matrix
  Paras.dists.rm<-Paras.dist[,!(row.names(Paras.dist) %in% cols.rem)]
  
  #Also remove corresponding rows in distance matrix
  Paras.dists.rm<-Paras.dist[!(row.names(Paras.dist) %in% cols.rem),]
  
  #Repeat for all distance matrices
  Host.dists.rm<-host.dist[,!(row.names(host.dist) %in% cols.rem)]
  Host.dists.rm<-host.dist[!(row.names(host.dist) %in% cols.rem),]
  
  Host.BL.dists.rm<-host.BL.dist[,!(row.names(host.BL.dist) %in% cols.rem)]
  Host.BL.dists.rm<-host.BL.dist[!(row.names(host.BL.dist) %in% cols.rem),]
  
  Geo.dists.rm<-Geo.dist[,!(row.names(Geo.dist) %in% cols.rem)]
  Geo.dists.rm<-Geo.dist[!(row.names(Geo.dist) %in% cols.rem),]
  
  maxtemp.dists.rm<-maxtemp.dist[,!(row.names(maxtemp.dist) %in% cols.rem)]
  maxtemp.dists.rm<-maxtemp.dist[!(row.names(maxtemp.dist) %in% cols.rem),]
  
  mintemp.dists.rm<-mintemp.dist[,!(row.names(mintemp.dist) %in% cols.rem)]
  mintemp.dists.rm<-mintemp.dist[!(row.names(mintemp.dist) %in% cols.rem),]
  
  precip.dists.rm<-precip.dist[,!(row.names(precip.dist) %in% cols.rem)]
  precip.dists.rm<-precip.dist[!(row.names(precip.dist) %in% cols.rem),]
  
  #Run a multiple regression on the edited distance matrices
  mod<-MRM(dist(Paras.dists.rm)~dist(Host.dists.rm)+dist(Host.BL.dists.rm)+
             dist(Geo.dists.rm)+dist(maxtemp.dists.rm) + dist(mintemp.dists.rm)+
             dist(precip.dists.rm),nperm =0)
  
  #Gather points for any desired plotting
  Host.Points<-as.vector(host.dist[lower.tri(Host.dists.rm)])
  paras.points<-as.vector(Paras.dist[lower.tri(Paras.dists.rm)])
  
  #Return list of mean connectivity and regression coefficients
  out<-list(Host.distances=Host.Points,parasite.distances=paras.points,
            Host.Phylo.R=mod$coef[1,1],Host.BL.Phylo.R=mod$coef[2,1],
            Geog.Dist.R=mod$coef[3,1],Maxtemp.Dist.R=mod$coef[4,1],
            Mintemp.Dist.R=mod$coef[5,1],Precip.Dist.R=mod$coef[6,1],
            R2=mod$r.squared[1],
            reg.names,
            Host.means,Host.BL.means,Paras.means)
}
#End function phylo.beta

#Function rep.phylo.beta will replicate phylo.beta, sampling new trees in each iteration
#requires all calls for phylo.beta & a 'reps' call for the number of iterations
rep.phylo.beta=function(host.comm,host.BL.comm,paras.comm,
                        host.phylogeny,host.BL.phylogeny,paras.phylogeny,
                        Geo.dist,maxtemp.dist,mintemp.dist,precip.dist,
                        calculation,reps,prop.remove){
  
  output<-data.frame(do.call(rbind,
                             replicate(reps,
                                       phylo.beta(host.comm,host.BL.comm,
                                                  paras.comm,host.phylogeny,
                                                  host.BL.phylogeny,paras.phylogeny,
                                                  Geo.dist,maxtemp.dist,mintemp.dist,
                                                  precip.dist,calculation,prop.remove),
                                       simplify=F)))
}
#End function rep.phylo.beta

#Function uniquenss.func will calculate uniqueness of each taxon in a site
#This function is not called directly but is called in ind.taxa.tidy
uniqueness.func<-function(x,dist,ntd){
  name.array<-array(na.omit(x))
  sample.dat.dist<-as.matrix(outer(name.array, name.array, 
                                   FUN=Vectorize(function(x,y) dist[paste(x),paste(y)])))
  rownames(sample.dat.dist)<-name.array->colnames(sample.dat.dist)
  diag(sample.dat.dist)<-NA
  
  #take mean community PD
  comm.mean<-sum(sample.dat.dist,na.rm=T)/length(!is.na(sample.dat.dist))
  if(ntd==FALSE){
    output<-colMeans(sample.dat.dist,na.rm=T) #take mean PD per individual taxon
    return(output / comm.mean) #standardise per community mean PD
  }
  if(ntd==TRUE){
    output<-colMeans(sample.dat.dist,na.rm=T) #take mean NTD per individual taxon
    return(output / mean(output)) #standardise per community mean NTD
  }
}
#end function uniqueness.func


#Function ind.taxa.raw applies uniqueness.func to each site separately
#This function is not called directly but is called in ind.taxa.tidy
ind.taxa.raw<-function(x,y,ntd,taxon){
  require(ape)
  tree<-sample(y,size=1)[[1]]
  phy.matrix<-cophenetic.phylo(tree)
  phy.matrix.scale<-phy.matrix/max(phy.matrix)
  output<-lapply(data.frame(t(x)),
                 FUN=function(x,y) uniqueness.func(x,phy.matrix.scale,ntd))
}
#end function ind.taxa.raw

#Function ind.taxa.tidy calculates median uniqueness of each sp. in each site
#This function is called, requiring the following inputs:
#nreps is the number of iterations, with a random tree in each iteration
#data is the data, with species as columns and sites as rows
#y is the distribution of phylogenetic trees that match species names
#ntd is a T/F statement to calculate nearest taxon (T) or mean pairwise uniqueness (F)
#taxon is the name to give to the species column (i.e "Parasite" or "Species" or "Host" etc..)
ind.taxa.tidy<-function(nreps,data,y,ntd,taxon){
  
  taxa.names<-colnames(data)
  x<-matrix(NA,nrow(data),ncol(data)); rownames(x)<-rownames(data)
  for(i in 1:nrow(data)){
    for(j in 1:ncol(data)){
      x[i,j]<-ifelse(data[i,j]==1,paste(taxa.names[j]),NA)
    }
  }
  
  dat<-do.call(rbind,replicate(nreps,unlist(ind.taxa.raw(x,y,ntd=ntd)),
                               simplify=F))
  require(matrixStats)
  taxa.unique<-data.frame(dat)
  
  #take median score
  taxa.unique<-data.frame(apply(taxa.unique, 2, FUN = median))
  taxa.unique$Region<-as.character(paste(row.names(taxa.unique)))
  row.names(taxa.unique)<-(1:length(taxa.unique[,1]))
  
  require(tidyr)
  taxa.unique<-separate(data = taxa.unique, col = Region, 
                        into = c("a", "b"), sep = "\\.")
  if(ntd==TRUE){
    colnames(taxa.unique)<-c(paste(taxon,"uniq.NT",
                                   sep = "_"),"SubRegion",paste(taxon))
    return(taxa.unique)
  }
  if(ntd==FALSE){
    colnames(taxa.unique)<-c(paste(taxon,"uniq",
                                   sep = "_"),"SubRegion",paste(taxon))
    return(taxa.unique)
  }
}
#end function ind.taxa.tidy


#Function comm.skew calculates skewness of communities
#requires x = taxon occurrences per region, y = tree distribution
comm.skew<-function(x,y){
  require(ape)
  require(e1071)
  tree<-sample(y,size=1)[[1]]
  phy.matrix<-cophenetic.phylo(tree) #calculate pairwise branch lengths
  phy.matrix.scale<-phy.matrix/(max(phy.matrix)) #scale branch lengths
  sample.dat<-t(x)
  sample.dat <- sample.dat[,colSums(sample.dat) > 0]
  name.array<-array(names(sample.dat))
  sample.dat.dist<-as.matrix(outer(name.array, name.array, 
                                   FUN=Vectorize(function(x,y) phy.matrix.scale[paste(x),paste(y)])))
  rownames(sample.dat.dist)<-name.array->colnames(sample.dat.dist)
  diag(sample.dat.dist)<-NA
  comm.skew<-skewness(as.vector(sample.dat.dist),na.rm=T,type=3) #calc community skewness
  return(comm.skew)
}
#end function comm.skew

#Function skew.tidy will tidy skew replicates
#requires x = comm.skew output, taxon = "colName for taxon" i.e. Plas, Haem, or HOst etc..
skew.tidy<-function(data,paras.trees,reps,taxon){
  require(matrixStats)
  b<-do.call(rbind,replicate(reps,unlist(apply(data[,],1,
            FUN=function(x,y) comm.skew(x,y=paras.trees))),simplify=F))
  taxa.unique<-data.frame(b)
  
  #take median score
  taxa.unique<-data.frame(apply(taxa.unique, 2, FUN = median))
  taxa.unique$Region<-as.character(paste(row.names(taxa.unique)))
  row.names(taxa.unique)<-(1:length(taxa.unique[,1]))
  
  require(tidyr)
  colnames(taxa.unique)<-c(paste(taxon,"Comm.skew",
                                 sep = "_"),"SubRegion")
  return(taxa.unique)
}
#end function skew.tidy

#Function phylorange.std to calculate host specificity STD (following Poulin)
phylorange.std<-function(data,trees){
  parasite<-unique(data$Lineage)
  randhost.tree<-sample(trees,size=1)[[1]]
  #scale branch lengths of the tree
  randhost.tree$edge.length=(randhost.tree$edge.length)/max(randhost.tree$edge.length)
  obs.std<-matrix(NA,length(parasite),2)
  colnames(obs.std)<-c("STD","Lineage")
  for(i in 1:length(parasite)){
    psub<-data[which(data$Lineage %in% paste(parasite[i])),] 
    host.obs<-unique(psub$Host)
    num.hosts<-length(unique(psub$Host))
    
    #only calculate host specificity for parasites infecting more than one host
    if(length(host.obs)>1){
      obs.phy.dis<-outer(host.obs,host.obs,
                         Vectorize(function(x,y)(cophenetic.phylo(randhost.tree)[x,y])))
      diag(obs.phy.dis)<-NA
      obs.numerator.mpd<-sum(obs.phy.dis[lower.tri(obs.phy.dis)])
      obs.denom.mpd<-num.hosts*(num.hosts-1)
      
      #take inverse of STD so d' and STD are interpreted in same direction
      obs.std[i,1]<-1 / ((2*(obs.numerator.mpd/obs.denom.mpd))+1)
    }
    
    #if a parasite only infects one host, attribute '1' for host specificity
    else{
      obs.std[i,1]<-1
    }
    obs.std[i,2]<-parasite[i]
  }
  
  obs.std
  return(obs.std)
}
#end phylorange.std function

# Modified functions for plotting phylo to map ----------------------------
# function depends on phytools (& dependencies) and maps (& dependencies)
# written by Liam J. Revell 2013

phylo.to.map<-function(tree,coords,rotate=TRUE,...){
  if(!inherits(tree,"phylo")) stop("tree should be an object of class \"phylo\".")
  # optional arguments
  if(hasArg(database)) database<-list(...)$database
  else database<-"world"
  if(hasArg(regions)) regions<-list(...)$regions
  else regions<-"."
  if(hasArg(plot)) plot<-list(...)$plot
  else plot<-TRUE
  if(hasArg(xlim)) xlim<-list(...)$xlim
  else xlim<-c(-180,180)
  if(hasArg(ylim)) ylim<-list(...)$ylim
  else ylim<-c(-90,90)
  # create a map
  map<-map(database,regions,xlim=xlim,ylim=ylim,plot=FALSE,fill=TRUE,resolution=0)
  # if rotate
  if(hasArg(type)) type<-list(...)$type else type<-"phylogram"
  if(hasArg(direction)) direction<-list(...)$direction else direction<-"downwards"
  if(rotate&&type=="phylogram") tree<-minRotate(tree,coords[,if(direction=="rightwards") 1 else 2])
  x<-list(tree=tree,map=map,coords=coords)
  class(x)<-"phylo.to.map"
  if(plot) plot.phylo.to.map(x,...)
  invisible(x)
}

# function to plot object of class "phylo.to.map"
# written by Liam J. Revell 2013, 2014, 2016

plot.phylo.to.map<-function(x,pan.col,type=c("phylogram","direct"),...){
  type<-type[1]
  if(class(x)=="phylo.to.map"){
    tree<-x$tree
    map<-x$map
    coords<-x$coords
  } else stop("x should be an object of class \"phylo.to.map\"")
  # get optional arguments
  if(hasArg(xlim)) xlim<-list(...)$xlim
  else xlim<-map$range[1:2]
  if(hasArg(ylim)) ylim<-list(...)$ylim
  else ylim<-map$range[3:4]
  if(hasArg(fsize)) fsize<-list(...)$fsize
  else fsize<-1.0
  if(hasArg(split)) split<-list(...)$split
  else split<-c(0.4,0.6)
  if(hasArg(psize)) psize<-list(...)$psize
  else psize<-1.0
  if(hasArg(cex.points)){ 
    cex.points<-list(...)$cex.points
    if(length(cex.points)==1) cex.points<-c(0.6*cex.points,cex.points)
  } else cex.points<-c(0.6*psize,psize)
  if(hasArg(mar)) mar<-list(...)$mar
  else mar<-rep(0,4)
  if(hasArg(asp)) asp<-list(...)$asp
  else asp<-1.0
  if(hasArg(ftype)) ftype<-list(...)$ftype
  else ftype<-"reg"
  ftype<-which(c("off","reg","b","i","bi")==ftype)-1
  if(!ftype) fsize=0 
  if(hasArg(from.tip)) from.tip<-list(...)$from.tip
  else from.tip<-FALSE
  if(hasArg(colors)) colors<-list(...)$colors
  else colors<-"red"
  if(length(colors)==1) rep(colors[1],2)->colors
  if(length(colors)==2&&type=="phylogram"){
    colors<-matrix(rep(colors,nrow(coords)),nrow(coords),2,byrow=TRUE)
    rownames(colors)<-rownames(coords)
  }
  if(length(colors)==nrow(coords)&&type=="phylogram"){   #colors editing
    colors<-matrix(colors)
    colors<-cbind(colors,rep("darkgrey",nrow(coords)))
    id<-tree$edge[tree$edge[,2]<=length(tree$tip.label),2]
    colors<-data.frame(cbind(colors,id))
    colors[,1]<-as.character(colors[,1])
  }
  if(hasArg(direction)) direction<-list(...)$direction
  else direction<-"downwards"
  if(hasArg(pch)) pch<-list(...)$pch
  else pch<-21
  if(length(pch)==1) pch<-rep(pch,2)
  if(hasArg(lwd)) lwd<-list(...)$lwd
  else lwd<-c(2,1)
  if(length(lwd)==1) lwd<-rep(lwd,2)
  if(hasArg(lty)) lty<-list(...)$lty
  else lty<-"dashed"
  if(hasArg(pts)) pts<-list(...)$pts
  else pts<-TRUE
  # recompute ylim or xlim to leave space for the tree
  if(type=="phylogram"){
    if(direction=="downwards"){
      if(!ftype) ylim<-c(ylim[1],ylim[2]+0.03*diff(ylim))
      ylim<-c(ylim[1],ylim[2]+
                split[1]/split[2]*(ylim[2]-ylim[1]))
    } else if(direction=="rightwards"){
      if(!ftype) xlim<-c(xlim[1]-0.03*diff(xlim),xlim[2])
      xlim<-c(xlim[1]-split[1]/split[2]*(xlim[2]-xlim[1]),xlim[2])
    }
  }
  # open & size a new plot
  par(mar=mar)
  plot.new()
  plot.window(xlim=xlim,ylim=ylim,asp=asp)
  map(map,add=TRUE,fill=TRUE,col="black",mar=rep(0,4),bg="gray90")  #change colour of map
  if(type=="phylogram"){
    ## preliminaries
    #cw<-reorder(tree,"cladewise")
    cw<-tree
    n<-Ntip(cw)
    if(direction=="downwards"){
      # plot a white rectangle
      dx<-abs(diff(xlim))
      rect(xlim[1]-1.04*dx,ylim[2]-split[1]*(ylim[2]-ylim[1]),
           xlim[2]+1.04*dx,ylim[2],col=pan.col,border=pan.col)
      # rescale tree so it fits in the upper half of the plot
      # with enough space for labels
      pdin<-par()$din[2]
      sh<-(fsize*strwidth(paste(" ",cw$tip.label,sep=""))+
             0.3*fsize*strwidth("W"))*(par()$din[1]/par()$din[2])*
        (diff(par()$usr[3:4])/diff(par()$usr[1:2]))
      cw$edge.length<-cw$edge.length/max(nodeHeights(cw))*
        (split[1]*(ylim[2]-ylim[1])-max(sh))
      pw<-reorder(cw,"postorder") ## post-order
      x<-vector(length=n+cw$Nnode)
      x[cw$edge[cw$edge[,2]<=n,2]]<-0:(n-1)/(n-1)*(xlim[2]-xlim[1])+xlim[1]
      nn<-unique(pw$edge[,1])
      # compute horizontal position of each edge
      for(i in 1:length(nn)){
        xx<-x[pw$edge[which(pw$edge[,1]==nn[i]),2]]
        x[nn[i]]<-mean(range(xx))
      }
      # compute start & end points of each edge
      Y<-ylim[2]-nodeHeights(cw)
      
      # plot coordinates & linking lines
      coords<-coords[,c(2,1,3)]
      for(i in 1:n) lines(c(x[i],coords[,1][coords[,3]==i]),
                          c(Y[which(cw$edge[,2]==i),2]-
                              if(from.tip) 0 else sh[i],coords[,2][coords[,3]==i]),
                          col=colors[,1][colors$id==i],
                          lty=lty,lwd=lwd[2])
      points(coords,pch=pch,cex=cex.points[2.5],bg="gray87")
      # plot vertical edges
      for(i in 1:nrow(Y)) lines(rep(x[cw$edge[i,2]],2),Y[i,],
                                lwd=lwd[1],lend=2)
      # plot horizontal relationships
      for(i in 1:cw$Nnode+n) 
        lines(range(x[cw$edge[which(cw$edge[,1]==i),2]]),
              Y[which(cw$edge[,1]==i),1],lwd=lwd[1],lend=2)
      # plot tip labels
      for(i in 1:n){ 
        if(ftype) text(x[i],Y[which(cw$edge[,2]==i),2],
                       paste(" ",sub("_"," ",cw$tip.label[i]),sep=""), ###editing points
                       pos=4,offset=c(0,1),
                       srt=-90,cex=fsize,font=ftype)
        if(pts) points(x[i],Y[which(cw$edge[,2]==i),2],pch=21,
                       bg=colors[i,1],
                       cex=0)
      }
      PP<-list(type="phylogram",use.edge.length=TRUE,node.pos=1,
               show.tip.label=if(ftype) TRUE else FALSE,show.node.label=FALSE,
               font=ftype,cex=fsize,adj=0,srt=0,no.margin=FALSE,
               label.offset=fsize*strwidth(" ")/(par()$usr[2]-
                                                   par()$usr[1])*(par()$usr[4]-par()$usr[3]),
               x.lim=par()$usr[1:2],y.lim=par()$usr[3:4],
               direction=direction,tip.color="black",Ntip=Ntip(cw),
               Nnode=cw$Nnode,edge=cw$edge,xx=x,yy=sapply(1:(Ntip(cw)+
                                                               cw$Nnode),function(x,y,z) y[match(x,z)],y=Y,z=cw$edge))
    } else {
      dy<-abs(diff(ylim))
      rect(xlim[1],ylim[1],xlim[1]+split[1]*(xlim[2]-
                                               xlim[1]),ylim[2],col="white",border="white")
      sh<-fsize*strwidth(paste(" ",cw$tip.label,sep=""))+
        0.2*fsize*strwidth("W")
      cw$edge.length<-cw$edge.length/max(nodeHeights(cw))*
        (split[1]*(xlim[2]-xlim[1])-max(sh))
      pw<-reorder(cw,"postorder")
      y<-vector(length=n+cw$Nnode)
      y[cw$edge[cw$edge[,2]<=n,2]]<-0:(n-1)/(n-1)*(ylim[2]-ylim[1])+ylim[1]
      nn<-unique(pw$edge[,1])
      for(i in 1:length(nn)){
        yy<-y[pw$edge[which(pw$edge[,1]==nn[i]),2]]
        y[nn[i]]<-mean(range(yy))
      }
      H<-nodeHeights(cw)
      X<-xlim[1]+H
      coords<-coords[,2:1]
      for(i in 1:n) lines(c(X[which(cw$edge[,2]==i),2]+
                              if(from.tip) 0 else sh[i],coords[i,1]),
                          c(y[i],coords[i,2]),col=colors[i,1],lty=lty,lwd=lwd[2])
      points(coords,pch=pch,cex=cex.points[2.5],bg=colors[cw$tip.label,2])
      for(i in 1:nrow(X)) lines(X[i,],rep(y[cw$edge[i,2]],2),lwd=lwd[1],lend=2)
      for(i in 1:cw$Nnode+n) lines(X[which(cw$edge[,1]==i),1],
                                   range(y[cw$edge[which(cw$edge[,1]==i),2]]),lwd=lwd[1],lend=2)
      for(i in 1:n){
        if(ftype) text(X[which(cw$edge[,2]==i),2],y[i],
                       paste(" ",sub("_"," ",cw$tip.label[i]),sep=""),
                       pos=4,offset=0.1,cex=fsize,font=ftype)
        if(pts) points(X[which(cw$edge[,2]==i),2],y[i],
                       pch=21,bg=colors[cw$tip.label,][i,2],cex=cex.points[1])
      }
      PP<-list(type="phylogram",use.edge.length=TRUE,node.pos=1,
               show.tip.label=if(ftype) TRUE else FALSE,show.node.label=FALSE,
               font=ftype,cex=fsize,adj=0,srt=0,no.margin=FALSE,label.offset=0.1,
               x.lim=par()$usr[1:2],y.lim=par()$usr[3:4],
               direction=direction,tip.color="black",Ntip=Ntip(cw),Nnode=cw$Nnode,
               edge=cw$edge,xx=sapply(1:(Ntip(cw)+cw$Nnode),
                                      function(x,y,z) y[match(x,z)],y=X,z=cw$edge),yy=y)
    }
    assign("last_plot.phylo",PP,envir=.PlotPhyloEnv)
  } else if(type=="direct"){
    phylomorphospace(tree,coords[,2:1],add=TRUE,label="horizontal",
                     node.size=c(0,psize),lwd=lwd[2],control=list(col.node=
                                                                    setNames(rep(colors[2],max(tree$edge)),1:max(tree$edge)),
                                                                  col.edge=setNames(rep(colors[1],nrow(tree$edge)),tree$edge[,2])),
                     ftype=c("off","reg","b","i","bi")[ftype+1],fsize=fsize)
  }
}


## rotates all nodes to try and match tip an ordering 
## written by Liam J. Revell 2013, 2015
minRotate<-function(tree,x,...){
  if(hasArg(print)) print<-list(...)$print
  else print<-TRUE
  tree<-reorder(tree)
  nn<-1:tree$Nnode+Ntip(tree)
  x<-x[tree$tip.label]
  for(i in 1:tree$Nnode){
    tt<-read.tree(text=write.tree(rotate(tree,nn[i])))
    oo<-sum(abs(order(x[tree$tip.label])-1:length(tree$tip.label)))
    pp<-sum(abs(order(x[tt$tip.label])-1:length(tt$tip.label)))
    if(oo>pp) tree<-tt
    if(print) message(paste("objective:",min(oo,pp)))
  }
  attr(tree,"minRotate")<-min(oo,pp)
  return(tree)
}

print.phylo.to.map<-function(x,...){
  cat("Object of class \"phylo.to.map\" containing:\n\n")
  cat(paste("(1) A phylogenetic tree with",Ntip(x$tree),"tips and",x$tree$Nnode,"internal nodes.\n\n",sep=" "))
  cat("(2) A geographic map with range:\n")
  cat(paste("     ",paste(round(x$map$range[3:4],2),collapse="N, "),"N\n",sep=""))
  cat(paste("     ",paste(round(x$map$range[1:2],2),collapse="W, "),"W.\n\n",sep=""))
  cat(paste("(3) A table containing",nrow(x$coords),"geographic coordinates.\n\n"))
}

rotate <- function(phy, node, polytom = c(1,2)){
  # load DESCENDANTS function
  DESCENDANTS <- function(tree, node){
    tips <- length(tree$tip.label)
    x <- tree$edge[,2][tree$edge[,1] == node]
    while(max(x) > tips){
      x <- x[x > tips] 
      for(h in 1:length(x)) tree$edge <- tree$edge[!tree$edge[,2] == x[h],]
      for(i in 1:length(x)) tree$edge[,1][tree$edge[,1] == x[i]] <- node
      x <- tree$edge[,2][tree$edge[,1] == node] 
    }
    x  
  }
  # function starts here	
  # definitions
  if (!inherits(phy, "phylo")) # is phy of class phylo?
    stop("object \"phy\" is not of class \"phylo\"")
  nb.tips <- length(phy$tip.label) # number of tiplabels
  max.int.node <- phy$Nnode+nb.tips # number of last internal node
  nb.edges <- dim(phy$edge)[1] # number of branches
  if (length(node) == 2){ # get MRCA if tips are given for node
    if (mode(node) == "character"){
      if (any(!node %in% phy$tip.label)) # do tiplabels correspond
        stop("object \"node\" contains tiplabels not present in object \"phy\"")
      tips <- cbind(phy$tip.label, 1:nb.tips)
      node[1] <- tips[,2][tips[,1] == node[1]]
      node[2] <- tips[,2][tips[,1] == node[2]]
      node <- as.numeric(node)
    }
    if (any(!node %in% 1:nb.tips)) # is phy of class phylo?
      stop("object \"node\" does not contain terminal nodes")
    node <- getMRCA(phy, node)
  }
  if (node  <= nb.tips || node > max.int.node) # is node really internal?
    stop("object \"node\" is not an internal node of object \"phy\"")  
  with.br.length <- !is.null(phy$edge.length) # does phy contain brlength?
  G <- cbind(phy$edge, 1:(length(phy$edge)/2)) 
  N <- phy$edge[phy$edge[,1] == node]
  N <- N[N != node]
  if (length(N) > 2) N <- N[polytom] 
  CLADE1 <- N[1]
  CLADE2 <- N[2]
  # do clades comprise interior nodes?
  if (CLADE1 > nb.tips) CLADE11 <- DESCENDANTS(phy, CLADE1)
  if (CLADE2 > nb.tips) CLADE22 <- DESCENDANTS(phy, CLADE2)
  # calculate inidices of clades in phy.edge
  if (CLADE1 > nb.tips){
    c1 <- G[,3][G[,2] == CLADE1]
    c2 <- G[,3][G[,2] == max(CLADE11)]
  } else {
    c1 <- G[,3][G[,2] == CLADE1]
    c2 <- G[,3][G[,2] == CLADE1]
  }
  if (CLADE2 > nb.tips){
    c3 <- G[,3][G[,2] == CLADE2]
    c4 <- G[,3][G[,2] == max(CLADE22)]	
  } else {
    c3 <- G[,3][G[,2] == CLADE2]
    c4 <- G[,3][G[,2] == CLADE2]
  }
  
  # create new phy$edge and  phy$edge.length
  if (c2+1 == c3){
    if (c1 == 1 && c4 != nb.edges){
      phy$edge <- rbind(phy$edge[c3:c4,], phy$edge[c1:c2,], phy$edge[(c4+1):nb.edges,])
      if (with.br.length)
        phy$edge.length <- c(phy$edge.length[c3:c4], phy$edge.length[c1:c2], phy$edge.length[(c4+1):nb.edges])
    }
    if (c1 !=1 && c4 == nb.edges){
      phy$edge <- rbind(phy$edge[1:(c1-1),], phy$edge[c3:c4,], phy$edge[c1:c2,])
      if (with.br.length)
        phy$edge.length <- c(phy$edge.length[1:(c1-1)], phy$edge.length[c3:c4], phy$edge.length[c1:c2])
    }
    if (c1 !=1 && c4 != nb.edges){
      phy$edge <- rbind(phy$edge[1:(c1-1),], phy$edge[c3:c4,], phy$edge[c1:c2,], phy$edge[(c4+1):nb.edges,])
      if (with.br.length)
        phy$edge.length <- c(phy$edge.length[1:(c1-1)], phy$edge.length[c3:c4], phy$edge.length[c1:c2], phy$edge.length[(c4+1):nb.edges])
    }
    if (c1 ==1 && c4 == nb.edges){
      phy$edge <- rbind(phy$edge[c3:c4,], phy$edge[c1:c2,])
      if (with.br.length)
        phy$edge.length <- c(phy$edge.length[c3:c4], phy$edge.length[c1:c2])
    }
  }
  else {
    if (c1 == 1 && c4 != nb.edges){
      phy$edge <- rbind(phy$edge[c3:c4,], phy$edge[(c2+1):(c3-1),], phy$edge[c1:c2,], phy$edge[(c4+1):nb.edges,])
      if (with.br.length)
        phy$edge.length <- c(phy$edge.length[c3:c4], phy$edge.length[(c2+1):(c3-1)], phy$edge.length[c1:c2], phy$edge.length[(c4+1):nb.edges])
    }
    if (c1 !=1 && c4 == nb.edges){
      phy$edge <- rbind(phy$edge[1:(c1-1),], phy$edge[c3:c4,], phy$edge[(c2+1):(c3-1),], phy$edge[c1:c2,])
      if (with.br.length)
        phy$edge.length <- c(phy$edge.length[1:(c1-1)], phy$edge.length[c3:c4], phy$edge.length[(c2+1):(c3-1)], phy$edge.length[c1:c2])
    }
    if (c1 !=1 && c4 != nb.edges){
      phy$edge <- rbind(phy$edge[1:(c1-1),], phy$edge[c3:c4,], phy$edge[(c2+1):(c3-1),], phy$edge[c1:c2,], phy$edge[(c4+1):nb.edges,])
      if (with.br.length)
        phy$edge.length <- c(phy$edge.length[1:(c1-1)], phy$edge.length[c3:c4], phy$edge.length[(c2+1):(c3-1)], phy$edge.length[c1:c2], phy$edge.length[(c4+1):nb.edges])
    }
    if (c1 ==1 && c4 == nb.edges){
      phy$edge <- rbind(phy$edge[c3:c4,], phy$edge[(c2+1):(c3-1),], phy$edge[c1:c2,])
      if (with.br.length)
        phy$edge.length <- c(phy$edge.length[c3:c4], phy$edge.length[(c2+1):(c3-1)], phy$edge.length[c1:c2])
    }
  }
  S <- write.tree(phy)
  phy <- if (!with.br.length) clado.build(S) else tree.build(S)
  phy
}
