library(plyr)
library(ade4)
library(ape)

#Load new environment to store functions needed for analyses
e = new.env() 
with(e,source("Comm.phylo.functions.R",local=TRUE)) 
attach(e) 

# Convert longs and lats to geographic distance matrix (kms) -----------------------------
#Calculate central latitude and longitude for each SubRegion
Dat  <- read.csv("Regional.lats.longs.csv", header=TRUE, 
                 stringsAsFactors = FALSE)
cent.points<-ddply(Dat,.(SubRegion),summarise,
                   Lat=mean(unique(Latitude),na.rm=T),
                   Long=mean(unique(Longitude),na.rm=T) )
cent.points<-cent.points[complete.cases(cent.points),]

#Calculate geographic distances for sample regions using CalcDists function
geo.dists <- CalcDists(cent.points[,2:3])
rownames(geo.dists)<-colnames(geo.dists)<-cent.points$SubRegion

#Change to distances in kilometers
geo.dists<-geo.dists/1000
rm(cent.points)

# Calculate climate Gower’s Distances -------------------------------------
#Read in climate variable data
climate.dat<-read.csv("sites_WorldClim_161216.csv",header=TRUE, stringsAsFactors = FALSE)

#Remove sites with NAs, take mean of all numeric columns for SubRegions
climate.dat<-climate.dat[complete.cases(climate.dat),]
climate.dat<-ddply(climate.dat,.(SubRegion),numcolwise(mean))
climate.dat <-rbind(climate.dat, climate.dat[1,])
climate.dat[[28, 1]] <- "Australia_CEast_HeronIsland"
climate.dat <-rbind(climate.dat, climate.dat[21,])
climate.dat[[29, 1]] <- "Micronesia_central" 

#Calculate precipitation Gower's distance matrix
#Use annual precip (12), precip of wettest quarter (16) & precip of driest quarter (17)
precip.vars<-data.frame(climate.dat[,c("WorldClim_bio.12","WorldClim_bio.16",
                                       "WorldClim_bio.17")])
rownames(precip.vars)<-(climate.dat$SubRegion) 
ktab1 <- ktab.list.df(list(precip.vars))
dist.precip <- dist.ktab(ktab1, type="Q", "scaledBYrange")

#Calculate max temp Gower's distance matrix
#Use max temp warmest month (5) and mean temp warmest quarter (10)
maxtemp.vars<-data.frame(climate.dat[,c("WorldClim_bio.5","WorldClim_bio.10")])
rownames(maxtemp.vars)<-(climate.dat$SubRegion) 
ktab1 <- ktab.list.df(list(maxtemp.vars))
dist.maxtemp <- dist.ktab(ktab1, type="Q", "scaledBYrange")

#Calculate min temp Gower's distance matrix
#Use min temp coldest month (6) and mean temp coldest quarter (11)
mintemp.vars<-data.frame(climate.dat[,c("WorldClim_bio.6","WorldClim_bio.11")])
rownames(mintemp.vars)<-(climate.dat$SubRegion) 
ktab1 <- ktab.list.df(list(mintemp.vars))
dist.mintemp <- dist.ktab(ktab1, type="Q", "scaledBYrange")

maxtemp.dist<-as.matrix(dist.maxtemp)
mintemp.dist<-as.matrix(dist.mintemp)
precip.dist<-as.matrix(dist.precip)

rm(precip.vars,maxtemp.vars,mintemp.vars,ktab1,
   dist.maxtemp,dist.mintemp,dist.precip,climate.dat)

# Read in host and parasite community data and phylogenies ------------------------
plas.comm<-as.matrix(read.csv("Plas_comm.data.csv",header=TRUE, 
                              stringsAsFactors = FALSE))
haem.comm<-as.matrix(read.csv("Haem_comm.data.csv",header=TRUE, 
                              stringsAsFactors = FALSE))
host.finite.comm<-as.matrix(read.csv("Host.finite_comm.data.csv",
                                     header=TRUE, stringsAsFactors = FALSE))
host.inclusive.comm<-as.matrix(read.csv("Host.inclusive_comm.data.csv",
                                     header=TRUE, stringsAsFactors = FALSE))

#Read in host and parasite phylogeny distributions
paras.trees<-read.nexus("Regional_Paras_trees.trees")
bird.trees<-read.nexus("Regional_Host_trees.tre")
bird.BL.trees<-read.nexus("Regional_Host_BL_trees.tre")


# Calculate community phylo similarities and MRM regressions ------------------------
#Function rep.phylo.beta calculates phylogenetic turnover for hosts and parasites
#Then calculates average pairwise turnover for each site as an index of connectivity
#Fincally, the function runs an MRM regression, storing coefficients and Rsquared
#Requires: host finite comm data, host inclusive comm data, parasite comm data
#host finite phylogenies, host inclusive phylogenies, parasite phylogenies
#geographic.dists, climate.dists1, climate.dists2, climate.dists3
#'calculation' (either 'cd' or 'nt')
#'reps' (number of iterations to perform, with random tree in each iteration)
#'prop.remove' (percent species to remove from well sampled sites, varies from 0 to x)
#Ignore warnings when replicating, can't seem to get rid of them
plas.phylocom<-rep.phylo.beta(host.finite.comm,host.inclusive.comm,
                              plas.comm,bird.trees,bird.BL.trees,paras.trees,geo.dists,
                              maxtemp.dist,mintemp.dist,precip.dist,"cd",1000,0.3)

haem.phylocom<-rep.phylo.beta(host.finite.comm,host.inclusive.comm,
                              haem.comm,bird.trees,bird.BL.trees,paras.trees,geo.dists,
                              maxtemp.dist,mintemp.dist,precip.dist,"cd",1000,0.3)

# Gather median community distances as an index of phylosimilarity --------
#For community phylosimilarity, higher values = more unique
#Unlist the rep.phylo.beta iterations and gather posterior samples
plas.phylo.means<-cbind(unlist(plas.phylocom[[10]]),
                        unlist(plas.phylocom[[11]]),
                        unlist(plas.phylocom[[12]]),
                        unlist(plas.phylocom[[13]]))
plas.phy.regs<-as.vector(as.character(plas.phylo.means[,1]))
host.phy.means<-as.numeric(plas.phylo.means[,2])
host.BL.phy.means<-as.numeric(plas.phylo.means[,3])
plas.phy.means<-as.numeric(plas.phylo.means[,4])

#Take median connectivity for each Region
plas.phy<-data.frame(SubRegion=plas.phy.regs,Host.phylo.dist=host.phy.means,
                     Host.inclusive.phylo.dist=host.BL.phy.means,
                     Plas.phylo.dist=plas.phy.means)
plas.phy<-ddply(plas.phy,.(SubRegion),numcolwise(median))

#Repeat for Haemoproteus
haem.phylo.means<-cbind(unlist(haem.phylocom[[10]]),
                        unlist(haem.phylocom[[13]]))
haem.phy.regs<-as.vector(as.character(haem.phylo.means[,1]))
haem.phy.means<-as.numeric(haem.phylo.means[,2])

haem.phy<-data.frame(SubRegion=haem.phy.regs,Haem.phylo.dist=haem.phy.means)
haem.phy<-ddply(haem.phy,.(SubRegion),numcolwise(median))

#Merge community medians together and save file
paras.phy<-merge(plas.phy,haem.phy,all.x=T,all.y=T)
write.table(paras.phy,file="Community_phylo_connectivity.csv",row.names=T,sep=",")
rm(plas.phylo.means,plas.phy.regs,plas.phy.means,
   haem.phylo.means,haem.phy.regs,haem.phy.means,
   plas.phy,haem.phy)

# Gather quantiles of MRM regression coefficients and Rsquareds -------------------------
#Unlist model coefficients and Rsquared values to create posterior distributions
plas.phycom.coefs<-cbind(unlist(plas.phylocom[[3]]),
                         unlist(plas.phylocom[[4]]),
                         unlist(plas.phylocom[[5]]),
                         unlist(plas.phylocom[[6]]),
                         unlist(plas.phylocom[[7]]),
                         unlist(plas.phylocom[[8]]),
                         unlist(plas.phylocom[[9]]))

#Convert to numeric to make sure none are character strings
plas.host.beta<-as.numeric(plas.phycom.coefs[,1])
plas.host.BL.beta<-as.numeric(plas.phycom.coefs[,2])
plas.geo.beta<-as.numeric(plas.phycom.coefs[,3])
plas.maxtemp.beta<-as.numeric(plas.phycom.coefs[,4])
plas.mintemp.beta<-as.numeric(plas.phycom.coefs[,5])
plas.precip.beta<-as.numeric(plas.phycom.coefs[,6])
plas.r2<-as.numeric(plas.phycom.coefs[,7])

#Bind coefs and Rsquareds together and take medians
plas.comm.MRM<-data.frame(Host.phybeta.effect=plas.host.beta,
                          Host.BL.phybeta.effect=plas.host.BL.beta,
                          Geo.dist.effect=plas.geo.beta,
                          Maxtemp.dist.effect=plas.maxtemp.beta,
                          Mintemp.dist.effect=plas.mintemp.beta,
                          Precip.dist.effect=plas.precip.beta,
                          Rsquared=plas.r2)

plas.comm.MRM<-apply(plas.comm.MRM,2,function(x) quantile(x,probs=c(0.025,0.5,0.975)))
write.table(plas.comm.MRM,file="Plas_MRM_Results.csv",row.names=T,sep=",")
rm(plas.phylocom,plas.phycom.coefs,plas.host.beta,plas.geo.beta,
   plas.maxtemp.beta,plas.mintemp.beta,plas.precip.beta,plas.r2)

#Repeat for Haemoproteus
haem.phycom.coefs<-cbind(unlist(haem.phylocom[[3]]),
                         unlist(haem.phylocom[[4]]),
                         unlist(haem.phylocom[[5]]),
                         unlist(haem.phylocom[[6]]),
                         unlist(haem.phylocom[[7]]),
                         unlist(haem.phylocom[[8]]),
                         unlist(haem.phylocom[[9]]))
haem.host.beta<-as.numeric(haem.phycom.coefs[,1])
haem.host.BL.beta<-as.numeric(haem.phycom.coefs[,2])
haem.geo.beta<-as.numeric(haem.phycom.coefs[,3])
haem.maxtemp.beta<-as.numeric(haem.phycom.coefs[,4])
haem.mintemp.beta<-as.numeric(haem.phycom.coefs[,5])
haem.precip.beta<-as.numeric(haem.phycom.coefs[,6])
haem.r2<-as.numeric(haem.phycom.coefs[,7])

haem.comm.MRM<-data.frame(Host.phybeta.effect=haem.host.beta,
                          Host.BL.phybeta.effect=haem.host.BL.beta,
                          Geo.dist.effect=haem.geo.beta,
                          Maxtemp.dist.effect=haem.maxtemp.beta,
                          Mintemp.dist.effect=haem.mintemp.beta,
                          Precip.dist.effect=haem.precip.beta,
                          Rsquared=haem.r2)

haem.comm.MRM<-apply(haem.comm.MRM,2,function(x) quantile(x,probs=c(0.025,0.5,0.975)))
write.table(haem.comm.MRM,file="Haem_MRM_Results.csv",row.names=T,sep=",")
rm(haem.phylocom,haem.phycom.coefs,haem.host.beta,haem.geo.beta,
   haem.maxtemp.beta,haem.mintemp.beta,haem.precip.beta,haem.r2)
