library(lme4)
library(plyr)
library(ape)
library(adephylo)
library(phangorn)
library(MCMCglmm)
library(MuMIn)
library(coda)
library(coefplot2)
library(ggplot2)
library(mulTree)

uniq.dat<-read.csv("Paras.Host.Region.analysis.csv", 
                   header=TRUE, stringsAsFactors = FALSE)

# Read host trees and specify priors ------------------------------
#Read in host tree for the host phylogeny random term
bird.trees<-read.nexus("Regional_Host_trees.tre")
randbird.tree<-sample(bird.trees,size=1)[[1]]

#Flat prior, standard to try but needs long runs to reach stationarity
prior <- list(R = list(V = 1, nu = 0.02), 
              G = list(G1 = list(V = 1,nu = 0.002),
                       G2 = list(V = 1, nu = 0.002)))

#Parameter-expanded prior (i.e. co-called redudancy priors; following Hadfield's advice)
#Speeds up iterations and reduces unwanted autocorrelation
prior.pe <- list(R= list(V=1, nu = 1), 
                 G= list(G1 = list(V=1, nu = 1, 
                                   alpha.mu=0, alpha.V = 100),
                         G2 = list(V=1, nu=1,
                                   alpha.mu=0,alpha.V = 100)))

#Run models using multiple avian phylogenetic trees using MulTree ---------
####Need to change Source Code of get.table function in mulTree -----------
get.table.mulTree<-function (mcmc.file) 
{
  table_mcmc <- as.data.frame(mcmc.file$Sol)
  random_terms <- c("phylogenetic.variance", "SubRegion.variance","residual.variance")
  table_mcmc[random_terms[1]] <- as.vector(mcmc.file$VCV[,1])
  table_mcmc[random_terms[2]] <- as.vector(mcmc.file$VCV[,2])
  table_mcmc[random_terms[3]] <- as.vector(mcmc.file$VCV[,3])
  return(as.list(table_mcmc))
}

assignInNamespace("get.table.mulTree", get.table.mulTree, "mulTree") 
mulTree:::get.table.mulTree

#Drop unsampled hosts from all phylogenies
tips.to.drop<-randbird.tree$tip.label[-match(uniq.dat$Host,randbird.tree$tip.label)]
pruned.trees<-lapply(bird.trees,drop.tip,tip=(tips.to.drop))
class(pruned.trees)<-"multiPhylo"

#Convert all phylogenies to ultrametric using Grafen's method
ultra.trees<-lapply(pruned.trees,compute.brlen,method="Grafen")
class(ultra.trees)<-"multiPhylo"

#Bind tree list to data, change 'Host' to 'taxa' to avoid strange name replacements
#We will run models on 50 different trees here
uniq.dat$taxa<-uniq.dat$Host 
mulTree.data<-as.mulTree(data=uniq.dat,tree=ultra.trees[1:50],taxa="taxa",
                         rand.terms=~taxa+SubRegion)

#Check that random terms are allocated properly, double check names of variables
mulTree.data$random.terms
names(mulTree.data$data)

#Running the Multi-tree MCMCglmm -----------------------------------------
#Note this code runs two chains of a full model ON EACH TREE
#NOT by using different tree in each iteration
#Best to use larger thin value to avoid huge number of samples
my_parameters<-c(100000,300,10000) #iterations, sampling, burnin

my.formula<-log(Paras_uniq) ~ scale(Host.BL_uniq)*Paras.Genus + scale(Host_uniq)+ 
  scale(Paras.Host.Dprime) + scale(Paras.STD) + scale(Host.range) + scale(Simps.Div)

mulTree(mulTree.data=mulTree.data,formula=my.formula,prior = prior.pe,
        family = "gaussian", parameters=my_parameters,verbose=T, 
        DIC=TRUE, nodes="ALL", output="multi.model")

all.models<-read.mulTree("multi.model")

#Gather Autocorrelations of variables
auto.corrs<-list()
for(i in 1:length(names(all.models))){
  auto.corrs[[i]]<-autocorr.diag(as.mcmc(all.models[[i]]))
}
names(auto.corrs)<-names(all.models)
auto.corrs

#Gather Effective Sample sizes for variables
eff.samps<-list()
for(i in 1:length(names(all.models))){
  eff.samps[[i]]<-effectiveSize(as.mcmc(all.models[[i]]))
}
names(eff.samps)<-names(all.models)
eff.samps

#Gather Highest Posterior Density intervals of coefficients
summarised_results <- summary(all.models, use.hdr = T, cent.tend = mean)
multi.phy.coefs<-matrix(NA,length(names(all.models)),3)
rownames(multi.phy.coefs)<-names(all.models)
colnames(multi.phy.coefs)<-c("LowerCI","Mode","UpperCI")
for(i in 1:length(names(all.models))){
  multi.phy.coefs[i,]<-summarised_results[i,c(2,1,5)]
}
write.table(multi.phy.coefs,file="MCMCglmm.covariate.coefs.csv",sep=",")

#Remove models from directory to clear space once diagnosis is complete
file.remove(list.files(pattern="multi.model"))

#Gather Cond and Marg. Requared Values -----------------------------------
#Need to collect design matrix from a single model
#here we use a singel tree MCMCglmm model (called mod.pe)
load(file="MCMC_output_pred1.RData", verbose = FALSE)

#Need to thin the full sample list, gather 1,000 samples and calc Rsquareds
all.models<-lapply(all.models,sample,size=1000)

mFixed<-matrix(NA,length(mod.pe$X[,2]),length(all.models[[2]]))
varFixed<-matrix(NA,1,length(all.models[[2]]))
var.mar<-matrix(NA,1,length(all.models[[2]]))
var.cond<-matrix(NA,1,length(all.models[[2]]))
varTotal<-matrix(NA,1,length(all.models[[2]]))
varBeta1<-matrix(NA,1,length(all.models[[2]]))
varBeta2<-matrix(NA,1,length(all.models[[2]]))
varBeta7<-matrix(NA,1,length(all.models[[2]]))
varBeta8<-matrix(NA,1,length(all.models[[2]]))
varRan1<-matrix(NA,1,length(all.models[[2]]))
varRan2<-matrix(NA,1,length(all.models[[2]]))

for(i in 1:length(all.models[[2]])){
  
  #summarise slopes for fixed effects (sum(beta.i * design.matrix.i))
  mFixed[,i]<-(all.models[[2]][i] * mod.pe$X[,2])+
    (all.models[[3]][i] * mod.pe$X[,3])+
    (all.models[[4]][i] * mod.pe$X[,4])+
    (all.models[[5]][i] * mod.pe$X[,5])+
    (all.models[[6]][i] * mod.pe$X[,6])+
    (all.models[[7]][i] * mod.pe$X[,7])+
    (all.models[[8]][i] * mod.pe$X[,8])+
    (all.models[[9]][i] * mod.pe$X[,9])
  
  #compute variance of fixed effect slopes
  varFixed[,i]<-var(mFixed[,i])
  
  #compute total variance, including fixed, random and residuals
  varTotal[,i]<-varFixed[,i]+all.models[[10]][i]+all.models[[11]][i]+all.models[[12]][i]
  
  #fixed effects Rsquared (marginal)
  var.mar[,i]<-varFixed[,i]/(varTotal[,i])
  
  #total Rsquared (fixed + random; conditional)
  var.cond[,i]<-(varFixed[,i]+all.models[[10]][i]+all.models[[11]][i])/
    varTotal[,i]
  
  #partial Rsquared for Beta1
  varBeta1[,i]<-var(all.models[[2]][i] * mod.pe$X[,2])/varTotal[,i]
  
  #partial Rsquared for Beta2
  varBeta2[,i]<-var(all.models[[3]][i] * mod.pe$X[,3])/varTotal[,i]
  
  #partial Rsquared for Beta7
  varBeta7[,i]<-var(all.models[[8]][i] * mod.pe$X[,8])/varTotal[,i]
  
  #partial Rsquared for Beta8
  varBeta8[,i]<-var(all.models[[9]][i] * mod.pe$X[,9])/varTotal[,i]
  
  #partial Rsquared for Random1
  varRan1[,i]<-all.models[[10]][i]/varTotal[,i]
  
  #partial Rsquared for Random2
  varRan2[,i]<-all.models[[11]][i]/varTotal[,i]
}

#Gather HPD intervals and posterior modes of Rsquared values
#Marginal Rsquared
plot(density(var.mar))
(HPDinterval(as.mcmc(t(var.mar))))
(mean(as.mcmc(t(var.mar))))

#Conditional Rsquared
plot(density(var.cond))
(HPDinterval(as.mcmc(t(var.cond))))
(posterior.mode(as.mcmc(t(var.cond))))

#Partial Rsquared for Host.BL_uniq
(HPDinterval(as.mcmc(t(varBeta1))))
(posterior.mode(as.mcmc(t(varBeta1))))

#Partial Rsquared for Paras.Genus
(HPDinterval(as.mcmc(t(varBeta2))))
(posterior.mode(as.mcmc(t(varBeta2))))

#Partial Rsquared for Host comm.connectivity
(HPDinterval(as.mcmc(t(varBeta7))))
(posterior.mode(as.mcmc(t(varBeta7))))

#Partial Rsquared for Host.BL_uniq:Paras.Genus interaction
(HPDinterval(as.mcmc(t(varBeta8))))
(posterior.mode(as.mcmc(t(varBeta8))))

#Partial Rsquared for Phylogeny random term
(HPDinterval(as.mcmc(t(varRan1))))
(posterior.mode(as.mcmc(t(varRan1))))

#Partial Rsquared for SubRegion random term
(HPDinterval(as.mcmc(t(varRan2))))
(posterior.mode(as.mcmc(t(varRan2))))

# Calculate Effect Sizes --------------------------------------------------
Paras_uniq<-uniq.dat$Paras_uniq
Host.BL_uniq<-uniq.dat$Host.BL_uniq
Host_uniq<-uniq.dat$Host_uniq
Paras.Genus<-uniq.dat$Paras.Genus
Paras.Host.Dprime<-uniq.dat$Paras.Host.Dprime
Host.range<-uniq.dat$Host.range
SubRegion<-uniq.dat$SubRegion
Host<-uniq.dat$Host
Paras.STD<-uniq.dat$Paras.STD
Simps.div<-uniq.dat$Simps.Div

transforms<-c(1,sd(Host.BL_uniq),1,sd(Host_uniq),
              sd(Paras.Host.Dprime),sd(Paras.STD),sd(Host.range),
              sd(Simps.div),1)

a<-matrix(NA,length(transforms),2)
for(i in 1:length(transforms)){
  a[i,]<-(HPDinterval(as.mcmc(all.models[[i]], prob=0.95)))
}

b<-matrix(NA,length(transforms),1)
for(i in 1:length(transforms)){
  b[i,]<-(posterior.mode(as.mcmc(all.models[[i]])))
}

effect.sizes<-data.frame(exp((a/transforms)),mode=exp((b/transforms)))
rm(a,b)
colnames(effect.sizes)<-c("lower","upper","mode")
rownames(effect.sizes)<-rownames(multi.phy.coefs)[1:9]
write.table(effect.sizes,file="MCMCglmm.covariate.effectsizes.csv",sep=",")

#Percent increase in Paras_uniq per unit increase in Host.BL_uniq index
#For Haem (the reference category)
effect.sizes[2,]

#For Plas (< than for Haem based on neg. coef., need to subtract interaction coef)
(effect.sizes[2,]-effect.sizes[9,])

#Increase in Paras_uniq per unit increase in Host community connectivity
effect.sizes[8,]


# MCMCglmm (for single tree models) -----------------------------------------
# Create data vectors and analysis data frame, prune trees -----------------------------
#Data vectors
Paras_uniq<-uniq.dat$Paras_uniq
Host.BL_uniq<-uniq.dat$Host.BL_uniq
Host_uniq<-uniq.dat$Host_uniq
Paras.Genus<-uniq.dat$Paras.Genus
Paras.Host.Dprime<-uniq.dat$Paras.Host.Dprime
Host.range<-uniq.dat$Host.range
SubRegion<-uniq.dat$SubRegion
animal<-uniq.dat$Host #need an 'animal' column for using phylogenetic random term
Host<-uniq.dat$Host
Paras.STD<-uniq.dat$Paras.STD
Simps.div<-uniq.dat$Simps.Div

Data<-data.frame(Paras_uniq,Host.BL_uniq,Host_uniq,Paras.Genus,
                 Paras.Host.Dprime,Host.range,
                 SubRegion,animal,Host,Paras.STD,Simps.div)

#Prune species not represented in data (speeds up MCMCglmm estimation)
pruned.tree<-drop.tip(randbird.tree,
                      randbird.tree$tip.label[-match(uniq.dat$Host,
                                                     randbird.tree$tip.label)])

# MCMCglmm model specification --------------------------------------------
#Run full model with parameter-expanded priors
mod.pe <- MCMCglmm(log(Paras_uniq) ~ scale(Host.BL_uniq)*Paras.Genus + scale(Host_uniq)
                   + scale(Paras.Host.Dprime) + scale(Paras.STD) + scale(Host.range) 
                   + scale(Simps.div),
                   random = ~ animal+SubRegion,pedigree=pruned.tree,prior = prior.pe,
                   family = "gaussian", nitt = 1000000, burnin = 100000, thin=250,
                   verbose=T, DIC=TRUE, nodes="ALL",data = Data)


save(mod.pe, file="MCMC_output_pred1.RData")
load(file="MCMC_output_pred1.RData", verbose = FALSE)

#Run second chain to check for convergence and increase confidence in estimates
mod.pe.2 <- MCMCglmm(log(Paras_uniq) ~ scale(Host.BL_uniq)*Paras.Genus +scale(Host_uniq)
                     + scale(Paras.Host.Dprime) + scale(Paras.STD) + scale(Host.range)
                     + scale(Simps.div),
                     random = ~ animal+SubRegion,pedigree=pruned.tree,prior = prior.pe,
                     family = "gaussian", nitt = 1000000, burnin = 100000, thin=250,
                     verbose=T, DIC=TRUE, nodes="ALL",data = Data)
save(mod.pe.2, file="MCMC_output_pred1_chain2.RData")
load(file="MCMC_output_pred1_chain2.RData", verbose = FALSE)

#Run full model without phylogeny random term to compare influence of host phylogeny
mod.pe.host <- MCMCglmm(log(Paras_uniq) ~ scale(Host.BL_uniq)*Paras.Genus +scale(Host_uniq)
                        + scale(Paras.Host.Dprime) + scale(Paras.STD) + scale(Host.range)
                        + scale(Simps.div),
                        random = ~ SubRegion,pedigree=pruned.tree,prior = prior.pe,
                        family = "gaussian", nitt = 1000000, burnin = 100000, thin=250,
                        verbose=T, DIC=TRUE,data = Data)
save(mod.pe.host, file="MCMC_output.host_pred1.RData")
load(file="MCMC_output.host_pred1.RData", verbose = FALSE)

#Compare models to check if phylogeny random term improves fit
model.sel(mod.pe,mod.pe.host, rank="DIC")

# Assessing chain diagnostics ---------------------------------------------
#Plot both chains to visually check for stationarity and convergence
library(lattice)
xyplot(mcmc.list(mod.pe$Sol,mod.pe.2$Sol),col=c("red","darkgrey"))
xyplot(mcmc.list(mod.pe$VCV,mod.pe.2$VCV),col=c("red","darkgrey"))

#Calculate autocorrelations between samples. Values <0.1 are preferred
autocorr.diag(mcmc.list(mod.pe$Sol, mod.pe.2$Sol))
autocorr.diag(mcmc.list(mod.pe$VCV, mod.pe.2$VCV))

#Gelman-Rubrin diagnostic of chain convergence. Need values near 1
gelman.diag(mcmc.list(mod.pe$Sol, mod.pe.2$Sol),confidence = 0.95)
gelman.diag(mcmc.list(mod.pe$VCV, mod.pe.2$VCV),confidence = 0.95)

#Calculate effective sample sizes for variables. EFS >1,000 are preferred 
effectiveSize(mcmc.list(mod.pe$Sol, mod.pe.2$Sol))
effectiveSize(mcmc.list(mod.pe$VCV, mod.pe.2$VCV))

#Model summary
summary(mod.pe)

# Gather intervals for regression coefficients and plot ----------
#HPD intervals of fixed coefficients
p.int<-HPDinterval(as.mcmc(rbind(mod.pe$Sol, mod.pe.2$Sol)))
fixed.intervals<-cbind(p.int,data.frame(mode = posterior.mode(as.mcmc(rbind(mod.pe$Sol, mod.pe.2$Sol)))))
rm(p.int)

#HPD intervals of random coefficients
p.int<-HPDinterval(as.mcmc(rbind(mod.pe$VCV, mod.pe.2$VCV)))
rand.intervals<-cbind(p.int,data.frame(mode = posterior.mode(as.mcmc(rbind(mod.pe$VCV, mod.pe.2$VCV)))))
rm(p.int)

#Plot coefficients for phylogenetic and non-phylogenetic models
library(coefplot2)
vn=c("Host.BL uniqueness","Plas. vs Haem.","Host.finite uniqueness",
     "D' host specificity","STD host specificity","Host geographic range",
     "Community host connectivity","Host.BL uniq:Plas. vs Haem.")
coefplot2(mod.pe,varnames=vn,xlim=c(-0.1,0.1),main="Regression coefficients")
coefplot2(mod.pe.host, add=TRUE, col.pts="red",offset=0.1)
legend(x=0.045,y=8.8,c("Phylogenetic Model"),lty=1, bty="n",pch=16,lwd=1.2, col="black", cex=0.7)
legend(x=0.045,y=8.96,c("Standard Model"),lty=1,bty="n",pch=16,lwd=1.2,col="red",cex=0.7)

# Calculate marginal and conditional Rsquared values ----------------------
#Code following Shinichi Nakagawa and Holger Schielzeth (MethodsEcolEvol_2013)
mFixed<-matrix(NA,length(mod.pe$X[,2]),length(mod.pe$Sol[,1]))
varFixed<-matrix(NA,1,length(mod.pe$Sol[,1]))
var.mar<-matrix(NA,1,length(mod.pe$Sol[,1]))
var.cond<-matrix(NA,1,length(mod.pe$Sol[,1]))
varTotal<-matrix(NA,1,length(mod.pe$Sol[,1]))
varBeta1<-matrix(NA,1,length(mod.pe$Sol[,1]))
varBeta2<-matrix(NA,1,length(mod.pe$Sol[,1]))
varBeta7<-matrix(NA,1,length(mod.pe$Sol[,1]))
varBeta8<-matrix(NA,1,length(mod.pe$Sol[,1]))
varRan1<-matrix(NA,1,length(mod.pe$Sol[,1]))
varRan2<-matrix(NA,1,length(mod.pe$Sol[,1]))
varNull<-matrix(NA,1,length(mod.pe$Sol[,1]))

for(i in 1:length(mod.pe$Sol[,1])){
  
  #summarise slopes for fixed effects (sum(beta.i * design.matrix.i))
  mFixed[,i]<-(mod.pe$Sol[i,2] * mod.pe$X[,2])+
    (mod.pe$Sol[i,3] * mod.pe$X[,3])+
    (mod.pe$Sol[i,4] * mod.pe$X[,4])+
    (mod.pe$Sol[i,5] * mod.pe$X[,5])+
    (mod.pe$Sol[i,6] * mod.pe$X[,6])+
    (mod.pe$Sol[i,7] * mod.pe$X[,7])+
    (mod.pe$Sol[i,8] * mod.pe$X[,8])+
    (mod.pe$Sol[i,9] * mod.pe$X[,9])
  
  #compute variance of fixed effect slopes
  varFixed[,i]<-var(mFixed[,i])
  
  #compute total variance, including fixed, random and residuals
  varTotal[,i]<-varFixed[,i] + mod.pe$VCV[i,1] + mod.pe$VCV[i,2] + mod.pe$VCV[i,3]
  
  #fixed effects Rsquared (marginal)
  var.mar[,i]<-varFixed[,i]/(varTotal[,i])
  
  #total Rsquared (fixed + random; conditional)
  var.cond[,i]<-(varFixed[,i]+mod.pe$VCV[i,1] + mod.pe$VCV[i,2])/
    varTotal[,i]
  
  #partial Rsquared for Beta1
  varBeta1[,i]<-var(mod.pe$Sol[i,2] * mod.pe$X[,2])/varTotal[,i]
  
  #partial Rsquared for Beta2
  varBeta2[,i]<-var(mod.pe$Sol[i,3] * mod.pe$X[,3])/varTotal[,i]
  
  #partial Rsquared for Beta7
  varBeta7[,i]<-var(mod.pe$Sol[i,8] * mod.pe$X[,8])/varTotal[,i]
  
  #partial Rsquared for Beta8
  varBeta8[,i]<-var(mod.pe$Sol[i,9] * mod.pe$X[,9])/varTotal[,i]
  
  #partial Rsquared for Random1
  varRan1[,i]<-mod.pe$VCV[i,1]/varTotal[,i]
  
  #partial Rsquared for Random2
  varRan2[,i]<-mod.pe$VCV[i,2]/varTotal[,i]
  
}


#Gather HPD intervals and posterior modes of Rsquared values
#Marginal Rsquared
plot(density(var.mar))
(HPDinterval(as.mcmc(t(var.mar))))
(mean(as.mcmc(t(var.mar))))

#Conditional Rsquared
plot(density(var.cond))
(HPDinterval(as.mcmc(t(var.cond))))
(posterior.mode(as.mcmc(t(var.cond))))

#Partial Rsquared for Host.BL_uniq
plot(density(varBeta1))
(HPDinterval(as.mcmc(t(varBeta1))))
(posterior.mode(as.mcmc(t(varBeta1))))

#Partial Rsquared for Paras.Genus
plot(density(varBeta2))
(HPDinterval(as.mcmc(t(varBeta2))))
(posterior.mode(as.mcmc(t(varBeta2))))

#Partial Rsquared for Host comm.connectivity
plot(density(varBeta7))
(HPDinterval(as.mcmc(t(varBeta7))))
(posterior.mode(as.mcmc(t(varBeta7))))

#Partial Rsquared for Host.BL_uniq:Paras.Genus interaction
plot(density(varBeta8))
(HPDinterval(as.mcmc(t(varBeta8))))
(posterior.mode(as.mcmc(t(varBeta8))))

#Partial Rsquared for Phylogeny random term
plot(density(varRan1))
(HPDinterval(as.mcmc(t(varRan1))))
(posterior.mode(as.mcmc(t(varRan1))))

#Partial Rsquared for SubRegion random term
plot(density(varRan2))
(HPDinterval(as.mcmc(t(varRan2))))
(posterior.mode(as.mcmc(t(varRan2))))

# Calculate effect sizes for important predictors --------------------------
#First need to divide coefficient by variable sd account for scale
#Then need to exponentiate unscaled coefficients to back-transform from log link
transforms<-c(1,sd(Host.BL_uniq),1,sd(Host_uniq),
              sd(Paras.Host.Dprime),sd(Paras.STD),sd(Host.range),
              sd(Simps.div),1)
a<-(HPDinterval(as.mcmc(rbind(mod.pe$Sol, mod.pe.2$Sol)), prob=0.95))
b<-posterior.mode(as.mcmc(rbind(mod.pe$Sol, mod.pe.2$Sol)))
effect.sizes<-data.frame(exp((a/transforms)),mode=exp((b/transforms)))
rm(a,b)
colnames(effect.sizes)<-c("lower","upper","mode")

#Percent increase in Paras_uniq per unit increase in Host.BL_uniq index
#For Haem (the reference category)
effect.sizes[2,]

#For Plas (< than for Haem based on neg. coef., need to subtract interaction coef)
(effect.sizes[2,]-effect.sizes[9,])

#Increase in Paras_uniq per unit increase in Host community connectivity
effect.sizes[8,]
